//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using MindFusion.Scheduling.WinForms;
using System.Reflection;


namespace MindFusion.Scheduling.WinForms.Samples.CS.GoogleCalendar
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			calendar1.EnableDragCreate = true;

			calendar1.TimetableSettings.TimelineStyle.HeaderBrush = new MindFusion.Drawing.SolidBrush(headerFillColor);
			calendar1.TimetableSettings.TimelineStyle.HeaderTextColor = Color.Transparent;
			calendar1.TimetableSettings.TimelineStyle.LineColor = Color.Transparent;
			calendar1.TimetableSettings.TimelineStyle.TextColor = Color.Transparent;

			calendar1.TimetableSettings.Style.HeaderTextColor = headerTextColor;
			calendar1.TimetableSettings.Style.HeaderFont = headerFont;
			calendar1.TimetableSettings.Style.HeaderBrush = new MindFusion.Drawing.SolidBrush(headerFillColor);

			calendar1.TimetableSettings.CellStyle.BorderLeftColor = lineColor;
			calendar1.TimetableSettings.CellStyle.BorderTopColor = Color.Transparent;
			calendar1.TimetableSettings.CellStyle.BorderRightColor = lineColor;
			calendar1.TimetableSettings.CellStyle.BorderBottomColor = Color.Transparent; // this will be custom-drawn depending on cell
			calendar1.TimetableSettings.CellStyle.LineColor = lineColor;

			calendar1.TimetableSettings.HeaderDateFormat = "ddd M/d";
			calendar1.TimetableSettings.MainHeaderSize = 18;
			calendar1.TimetableSettings.InfoHeaderSize = 7;
			calendar1.TimetableSettings.InfoHeaderBrush = new MindFusion.Drawing.SolidBrush(headerFillColor);
			calendar1.TimetableSettings.ShowWorkTime = State.Disabled;
			calendar1.TimetableSettings.ShowDayHeader = State.Enabled;
			calendar1.TimetableSettings.ShowInfoHeader = State.Enabled;
			calendar1.TimetableSettings.ColumnBandSize = 0;

			calendar1.CustomDraw = CustomDrawElements.TimetableTimelineHourCell |
				CustomDrawElements.TimetableWholeDayHeader |
				CustomDrawElements.TimetableCell |
				CustomDrawElements.TimetableColumnHeader |
				CustomDrawElements.TimetableInfoHeader;
			calendar1.Draw += OnDraw;

			calendar1.ItemSettings.ShowClocks = ShowClocks.Never;

			calendar1.HiddenItemClick += calendar1_HiddenItemClick;
			calendar1.Paint += calendar1_Paint;
			calendar1.SizeChanged += calendar1_SizeChanged;
			calendar1.Click += calendar1_Click;

			calendar1.MouseDoubleClick += calendar1_MouseDoubleClick;

			Style personalEvents = new Style();
			personalEvents.TextColor = Color.FromArgb(0xFF, 0x80, 0x80, 0x80);
			personalEvents.BorderTopColor = Color.Goldenrod;
			personalEvents.BorderLeftColor = Color.Goldenrod;
			personalEvents.BorderBottomColor = Color.Goldenrod;
			personalEvents.BorderRightColor = Color.Goldenrod;
			personalEvents.LineColor = Color.Goldenrod;
			personalEvents.FillColor = Color.FromArgb(250, 200, 100);
			personalEvents.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleGoldenrod, 90);
			personalEvents.HeaderTextColor = Color.DarkGoldenrod;
			calendar1.ItemSettings.Style = personalEvents;
			calendar1.ItemSettings.SelectedItemStyle = personalEvents;

			// Disable all interactions while the info box is open
			calendar1.ItemModifying += calendar1_ItemModifying;
			calendar1.ItemInplaceEditStarting += calendar1_ItemInplaceEditStarting;
			calendar1.ItemSelecting += calendar1_ItemSelecting;

			calendar1.ShowToolTips = true;
			calendar1.ItemTooltipDisplaying += calendar1_ItemTooltipDisplaying;
			calendar1.MouseMove += calendar1_MouseMove;

			calendar1.ItemSettings.MoveBandSize = 0;
			calendar1.ItemSettings.ResizeBandSize = 0;

			calendar1.Selection.Style = SelectionStyle.None;
			calendar1.Selection.SelectedElementsStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(20, Color.Black));
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		void calendar1_MouseMove(object sender, MouseEventArgs e)
		{
			if (calendar1.GetItemAt(e.X, e.Y) != null)
				calendar1.Cursor = Cursors.Hand;
			else
				calendar1.Cursor = Cursors.Default;
		}

		void calendar1_ItemTooltipDisplaying(object sender, ItemTooltipEventArgs e)
		{
			if (e.Item.AllDayEvent)
			{
				e.Tooltip = e.Item.HeaderText + "\n" + e.Item.DescriptionText;
			}
			else
			{
				e.Tooltip = "";
			}
		}

		void calendar1_ItemSelecting(object sender, ItemConfirmEventArgs e)
		{
			if (infoColumn != -1)
				e.Confirm = false;
		}

		void calendar1_ItemInplaceEditStarting(object sender, ItemConfirmEventArgs e)
		{
			if (infoColumn != -1)
				e.Confirm = false;
		}

		void calendar1_ItemModifying(object sender, ItemModifyConfirmEventArgs e)
		{
			if (infoColumn != -1)
				e.Confirm = false;
		}

		private void calendar1_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			Point p = new Point(e.X, e.Y);
			for (int i = 0; i < calendar1.TimetableSettings.Dates.Count; i++)
			{
				Rectangle r = calendar1.GetElementBounds(CalendarElement.TimetableDayHeader, i);
				if (r.Contains(p))
				{
					Appointment app = new Appointment();
					app.StartTime = calendar1.TimetableSettings.Dates[i];
					app.EndTime = calendar1.TimetableSettings.Dates[i].AddDays(1);
					app.AllDayEvent = true;
					calendar1.Schedule.Items.Add(app);
				}
			}
		}

		private void calendar1_HiddenItemClick(object sender, DateEventArgs e)
		{
			// Find the index of the column
			int index = 0;
			foreach (DateTime date in calendar1.TimetableSettings.Dates)
			{
				if (date == e.Date)
					break;

				index++;
			}

			// If the same column is already displayed, ignore the click
			if (infoColumn == index)
				return;

			Rectangle headerBounds = calendar1.GetElementBounds(CalendarElement.TimetableColumnHeader, index);
			// Expand the rectangle to the bottom of the view
			headerBounds.Height = calendar1.Height - headerBounds.Y - 2;
			headerBounds.Y += calendar1.TimetableSettings.MainHeaderSize;
			headerBounds.Height -= calendar1.TimetableSettings.MainHeaderSize;
			currentColumnBounds = headerBounds;
			infoColumn = index;
			calendar1.Selection.Enabled = State.Disabled;
			calendar1.ItemSettings.ShowMoreItemsCue = State.Disabled;

			// Get all items in the column
			allItems = calendar1.Schedule.GetAllItems(e.Date, e.Date.AddDays(1));

			calendar1.Invalidate();

			ignoreNextClick = true;
		}

		private void calendar1_Paint(object sender, PaintEventArgs e)
		{
			// Creating a cascading style over Calendar.ItemSettings.Style.
			// All properties that are not explicitly set will propagate to
			// the parent style. Note, that the used constructor is not visible
			// because it was not meant to be used outside of the control.
			Style personalEvents = new Style(calendar1.ItemSettings.Style);
			// On the other hand you can simply use Calendar.ItemSettings.Style
			//Style personalEvents = calendar1.ItemSettings.Style;

			personalEvents.TextColor = Color.FromArgb(0xFF, 0x80, 0x80, 0x80);
			personalEvents.BorderTopColor = Color.Goldenrod;
			personalEvents.BorderLeftColor = Color.Goldenrod;
			personalEvents.BorderBottomColor = Color.Goldenrod;
			personalEvents.BorderRightColor = Color.Goldenrod;
			personalEvents.LineColor = Color.Goldenrod;
			personalEvents.FillColor = Color.FromArgb(250, 200, 100);
			personalEvents.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.PaleGoldenrod, 90);
			personalEvents.HeaderTextColor = Color.DarkGoldenrod;

			if (currentColumnBounds.HasValue)
			{
				using (Brush brush = new SolidBrush(Color.FromArgb(225, Color.White)))
					e.Graphics.FillRectangle(brush, currentColumnBounds.Value);
				e.Graphics.DrawRectangle(Pens.LightSteelBlue, currentColumnBounds.Value);

				Point p = currentColumnBounds.Value.Location;
				foreach (Item item in allItems)
				{
					// Instantiating from the ItemDrawContext class - it is done
					// through reflection because this class is not meant to be
					// instantiated directly.
					ConstructorInfo[] constructors = typeof(ItemDrawContext).GetConstructors(
						BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);

					MindFusion.Drawing.GdiGraphics gdiGraphics = new MindFusion.Drawing.GdiGraphics(e.Graphics);

					int padding = 2;
					ItemDrawContext context = constructors[0].Invoke(new object[]
					{
						calendar1,
						item,
						gdiGraphics,
						new Rectangle(p.X + padding, p.Y + padding, currentColumnBounds.Value.Width - 2 * padding, calendar1.ItemSettings.Size),
						personalEvents,
						true, // Is all-day event ?
						true, // Is horizontal ?
						true, // Starts here ?
						true, // Ends here ?
						false, // Is milestone?
						false, // Is pointed ?
						(MindFusion.Drawing.Brush)null // Explicit fill brush
					}) as ItemDrawContext;

					// Draw the item with its default appearance through the ItemDrawContext
					if (context != null)
						context.DrawDefault();

					p.Y += calendar1.ItemSettings.Size + padding;
				}
			}
		}

		private void calendar1_SizeChanged(object sender, EventArgs e)
		{
			currentColumnBounds = null;
			infoColumn = -1;
			calendar1.Selection.Enabled = State.Enabled;
			calendar1.ItemSettings.ShowMoreItemsCue = State.Enabled;
		}

		private void calendar1_Click(object sender, EventArgs e)
		{
			if (ignoreNextClick)
			{
				ignoreNextClick = false;
				return;
			}

			if (currentColumnBounds != null)
			{
				// If the click is within the currently displayed info box, do
				// not close the info box; instead try to hit-test the items
				// in the info box
				Point point = calendar1.PointToClient(Cursor.Position);
				if (currentColumnBounds.Value.Contains(point))
				{
					Point p = currentColumnBounds.Value.Location;
					foreach (Item item in allItems)
					{
						int padding = 2;

						Rectangle itemBounds = new Rectangle(p.X + padding, p.Y + padding,
							currentColumnBounds.Value.Width - 2 * padding, calendar1.ItemSettings.Size);

						if (itemBounds.Contains(point))
						{
							// TODO: Add custom logic here
							MessageBox.Show(item.HeaderText + " was clicked!");
						}

						p.Y += calendar1.ItemSettings.Size + padding;
					}

					return;
				}
			}

			currentColumnBounds = null;
			infoColumn = -1;
			calendar1.Selection.Enabled = State.Enabled;
			calendar1.ItemSettings.ShowMoreItemsCue = State.Enabled;

			calendar1.Invalidate();
		}

		private void OnDraw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.TimetableTimelineHourCell)
			{
				Rectangle bounds = e.Bounds;
				using (Brush brush = new SolidBrush(timelineFillColor))
					e.Graphics.FillRectangle(brush, bounds);
				bounds.Width -= 1;
				bounds.Height -= 1;
				using (Pen pen = new Pen(lineColor))
					//e.Graphics.DrawRectangle(pen, bounds);
					e.Graphics.DrawLine(pen, e.Bounds.Left, e.Bounds.Top, e.Bounds.Right - 1, e.Bounds.Top);

				DateTime time = e.Date.Add(e.StartTime);
				DateTime endTime = e.Date.Add(e.EndTime);

				using (Brush textBrush = new SolidBrush(timelineTextColor))
				{
					using (StringFormat format = new StringFormat(StringFormat.GenericTypographic))
					{
						format.LineAlignment = StringAlignment.Near;
						format.Alignment = StringAlignment.Far;
						e.Graphics.DrawString(string.Format("{0}{1}", time.ToString("h ").Trim(), time.Hour >= 12 ? "pm" : "am"), timelineFont, textBrush, bounds, format);
					}
				}

				TimeSpan firstVisibleTime = calendar1.GetFirstVisibleDate().TimeOfDay;
				if (time.TimeOfDay <= firstVisibleTime && firstVisibleTime < endTime.TimeOfDay)
				{
					// Bottom line
					using (Pen bottomPen = new Pen(frameColor))
						e.Graphics.DrawLine(bottomPen, e.Bounds.Left, e.Bounds.Top - 1, e.Bounds.Right - 1, e.Bounds.Top - 1);
				}
			}
			else if (e.Element == CustomDrawElements.TimetableWholeDayHeader)
			{
				bool isToday = e.Date == DateTime.Today;
				bool isTomorrow = e.Date.AddDays(-1) == DateTime.Today;

				using (Pen pen = new Pen(lineColor))
					e.Graphics.DrawLine(pen, e.Bounds.Right - 1, e.Bounds.Top, e.Bounds.Right - 1, e.Bounds.Bottom);

				if (isToday || isTomorrow)
				{
					// Fill
					if (isToday)
					{
						using (Brush brush = new SolidBrush(todayLightColor))
							e.Graphics.FillRectangle(brush, e.Bounds);
					}

					// Left/right side
					using (Pen todayPen = new Pen(todayDarkColor))
						e.Graphics.DrawLine(todayPen, e.Bounds.Left - 1, e.Bounds.Top, e.Bounds.Left - 1, e.Bounds.Bottom - 1);
				}
			}
			else if (e.Element == CustomDrawElements.TimetableInfoHeader)
			{
				// Bottom line
				using (Pen bottomPen = new Pen(frameColor))
					e.Graphics.DrawLine(bottomPen, e.Bounds.Left, e.Bounds.Bottom - 1, e.Bounds.Right - 1, e.Bounds.Bottom - 1);
			}
			else if (e.Element == CustomDrawElements.TimetableCell)
			{
				DateTime time = e.Date.Add(e.StartTime);
				DateTime endTime = e.Date.Add(e.EndTime);
				bool isToday = e.Date == DateTime.Today;
				bool isTomorrow = e.Date.AddDays(-1) == DateTime.Today;

				Pen pen = new Pen(lineColor);
				Pen todayPen = new Pen(todayDarkColor);

				if (time.Minute == 0)
					pen.DashStyle = DashStyle.Dot;
				else
					pen.DashStyle = DashStyle.Solid;

				// Fill
				if (isToday)
				{
					using (Brush brush = new SolidBrush(todayLightColor))
						e.Graphics.FillRectangle(brush, e.Bounds.X + 1, e.Bounds.Y + 1, e.Bounds.Width - 1, e.Bounds.Height - 1);
				}

				// Bottom side
				e.Graphics.DrawLine(pen, e.Bounds.Left, e.Bounds.Bottom - 1, e.Bounds.Right - 1, e.Bounds.Bottom - 1);

				if (time.Hour == 0 && time.Minute == 0)
				{
					// Top side
					pen.DashStyle = DashStyle.Solid;
					e.Graphics.DrawLine(pen, e.Bounds.Left, e.Bounds.Top, e.Bounds.Right - 1, e.Bounds.Top);
				}

				if (isToday || isTomorrow)
				{
					// Left side
					e.Graphics.DrawLine(todayPen, e.Bounds.Left, e.Bounds.Top, e.Bounds.Left, e.Bounds.Bottom - 1);
				}

				pen.Dispose();
				todayPen.Dispose();

				// Draw the current time indicator
				DateTime now = DateTime.Now;
				if (time <= now && now < endTime)
				{
					double relativePosition = ((double)now.Ticks - time.Ticks) / (endTime.Ticks - time.Ticks);

					int position = e.Bounds.Top + (int)(e.Bounds.Height * relativePosition);

					using (Pen nowPen = new Pen(nowColor, 2))
					{
						e.Graphics.DrawLine(nowPen, e.Bounds.Left + 3, position, e.Bounds.Right - 3, position);

						Point[] leftArrow = new Point[]
						{
							new Point(e.Bounds.Left + 1, position - 4),
							new Point(e.Bounds.Left + 1, position + 4),
							new Point(e.Bounds.Left + 4 + 1, position),
						};
						Point[] rightArrow = new Point[]
						{
							new Point(e.Bounds.Right - 1, position - 4),
							new Point(e.Bounds.Right - 1, position + 4),
							new Point(e.Bounds.Right - 4 - 1, position),
						};

						using (Brush nowBrush = new SolidBrush(nowColor))
						{
							e.Graphics.FillPolygon(nowBrush, leftArrow);
							e.Graphics.FillPolygon(nowBrush, rightArrow);
						}
					}
				}
			}
			else if (e.Element == CustomDrawElements.TimetableColumnHeader)
			{
				bool isToday = e.Date == DateTime.Today;

				if (isToday)
				{
					// Fill
					using (Brush brush = new SolidBrush(todayDarkColor))
						e.Graphics.FillRectangle(brush, e.Bounds);

					// Left side
					using (Pen pen = new Pen(todayDarkColor))
						e.Graphics.DrawLine(pen, e.Bounds.Left - 1, e.Bounds.Top, e.Bounds.Left - 1, e.Bounds.Bottom - 1);

					// Text
					using (Brush textBrush = new SolidBrush(e.Style.HeaderTextColor))
					{
						using (StringFormat format = new StringFormat(StringFormat.GenericTypographic))
						{
							format.LineAlignment = StringAlignment.Center;
							format.Alignment = StringAlignment.Center;
							e.Graphics.DrawString(e.Text, e.Style.HeaderFont, textBrush, e.Bounds, format);
						}
					}
				}
			}
		}


		private string _appPath;
		private Color lineColor = Color.FromArgb(0xDD, 0xDD, 0xDD);
		private Color timelineTextColor = Color.FromArgb(0x66, 0x66, 0x66);
		private Color timelineFillColor = Color.FromArgb(0xF6, 0xF9, 0xFC);
		private Font timelineFont = new Font("Arial", 11, FontStyle.Regular, GraphicsUnit.Pixel);
		private Color headerTextColor = Color.FromArgb(0x22, 0x00, 0xCC);
		private Font headerFont = new Font("Arial", 11, FontStyle.Regular, GraphicsUnit.Pixel);
		private Color headerFillColor = Color.FromArgb(0xE3, 0xE9, 0xFF);
		private Color todayDarkColor = Color.FromArgb(0xFA, 0xD1, 0x63);
		private Color todayLightColor = Color.FromArgb(0xFF, 0xF7, 0xD7);
		private Color nowColor = Color.FromArgb(0xFF, 0x7F, 0x6E);
		private Color frameColor = Color.FromArgb(0xBB, 0xCC, 0xFF);

		private Nullable<Rectangle> currentColumnBounds;
		private List<Item> allItems;

		private int infoColumn = -1;
		private bool ignoreNextClick = false;
	}
}
